/*  -*- c++ -*-  */
#ifndef MODIFIERISG_H
#define MODIFIERISG_H

#include "Modifier.h"
#include "iSGPAR.h"
#include "PSF.h"
#include "TRANS.h"
#include "Array.h"
#include <vector>

namespace ProtoMol {

  class iSGIntegrator;
  class ScalarStructure;
  class GenericTopology;

  //_________________________________________________________________ ModifierISG
  /**
     Modifies the Topology by ISG
  */
  class ModifierISG : public Modifier {

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    ModifierISG(iSGIntegrator *integrator, const PSF &psf, const iSGPAR &par);

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From class Modifier
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  public:
    virtual bool isInternal() const {return false;}

    void checkForStageCompletion();
    void setForcesAfterTransformation(GenericTopology *topo);
    void pickNewMolecule(GenericTopology *topo, ScalarStructure *myEnergies);
    
    //  If an initial XSC file is specified, read in the values
    //  of the extended system coordinates
    virtual void readXSCs(const std::string myFile, GenericTopology *topo);

    void indexBonds(GenericTopology *topo, const TRANS &trans);
    
  private:
    virtual void doExecute();
    virtual void doInitialize();
    virtual std::string doPrint()const{return std::string("ISG");};

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
    iSGIntegrator* myTheIntegrator;

  public:
    //_________________________________________________________________ TransformMap
    struct TransformMap {
      // This class contains information common to one type of atom.  
  
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      // Constructors, destructors, assignment
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      TransformMap(){}
      // Default constructor for one atom type's set of charges for each transformation stage.
  
      // constructor for iSGMD simulations that creates memory for all identities
      TransformMap(int i, int s) {
        old_charge.resize(ArraySizes(i)(i)(s));
        new_charge.resize(ArraySizes(i)(i)(s));
        alphaLJ.resize(ArraySizes(i)(i));
      }

      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      // My data members
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      Array<Real,3> old_charge;
      Array<Real,3> new_charge;
      // This atomtype's old_type and new_type charges for each transformation stage.
      Array<Real,2> alphaLJ;
      // This atomtype's alphaLJ parameter for a switch between two identities
    };

    //_________________________________________________________________ local storage elements
    std::vector<iSGPAR::Bond> bonds;
    std::vector<iSGPAR::Angle> angles;
    std::vector<iSGPAR::Dihedral> dihedrals;
    std::vector<iSGPAR::Improper> impropers;
    std::vector<iSGPAR::AtomType> atomTypes;
    std::vector<TransformMap> transformMaps;
  
  };

}
#endif /* MODIFIERISG_H */

