/*  -*- c++ -*-  */
#ifndef PDB_H
#define PDB_H

#include <string>
#include <vector>

#include "Vector3DBlock.h"
#include "Report.h"

namespace ProtoMol {
  //___________________________________________________________________PDB
  /**
   * PDB container holding PDBAtom's and coordinates
   */
  struct PDB {


    //___________________________________________________________________PDBAtom
    /**
     * This class holds data for a basic PDB element; excluding the 
     * coordinates (x,y,z), stored separately.
     */
    struct PDBAtom{
      PDBAtom();
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      // My data members
      //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
      PDBAtom(std::string elementType,  
	      int  elementNum,	
	      std::string elementName,  
	      std::string altLoc,	
	      std::string residueName,  
	      std::string chainID,	
	      int  residueNum,	
	      std::string insertionCode,
	      Real occupancy,	
	      Real tempFactor,	
	      int footnote,
	      std::string segID,	
	      std::string symbol,	
	      std::string charge,	
	      int hvyAtomGrpsize);

      std::string elementType;     ///< record_name        
      int         elementNum;	   ///< serial_size	   
      std::string elementName;     ///< atom_name	   
      std::string altLoc;	   ///< alternate_location 
      std::string residueName;     ///< residue_name	   
      std::string chainID;	   ///< chain_id	   
      int         residueNum;	   ///< residue_sequence   
      std::string insertionCode;   ///< insertion_code	   
      // x
      // y
      // z
      Real        occupancy;	   ///< occupancy	   
      Real        tempFactor;	   ///< temp_factor	   
      int         footnote;        ///< footnote number
      std::string segID;	   ///< seg_id		   
      std::string symbol;	   ///< element_symbol	   
      std::string charge;	   ///< charge		   
      int         hvyAtomGrpsize;  ///< ??? throw in zeros	   
      
      /// Record element start positions
      enum Start{
	S_RECORD_NAME    = 0,
	S_SERIAL         = 6,
	S_ATOM_NAME      = 12,
	S_ALT_LOC        = 16,
	S_RES_NAME       = 17,
	S_CHAIN_ID       = 21,
	S_RES_SEQ        = 22,
	S_I_CODE         = 26,
	S_X              = 30,
	S_Y              = 38,
	S_Z              = 46,
	S_OCCUP          = 54,
	S_TEMP           = 60,
	S_FOOT_NOTE      = 67,
	S_SEG_ID         = 72,
	S_ELEMENT_SYMBOL = 76,
	S_CHARGE         = 78
      };

      /// Record element length
      enum Length{
	L_RECORD_NAME      = 6,
	L_SERIAL           = 5,
	L_ATOM_NAME        = 4,
	L_ALT_LOC          = 1,
	L_RES_NAME         = 4,
	L_CHAIN_ID         = 1,
	L_RES_SEQ          = 4,
	L_I_CODE           = 1,
	L_X                = 8,
	L_Y                = 8,
	L_Z                = 8,
	L_OCCUP            = 6,
	L_TEMP             = 6,
	L_FOOT_NOTE        = 3,
	L_SEG_ID           = 4,
	L_ELEMENT_SYMBOL   = 2,
	L_CHARGE           = 2
      };


      friend Report::MyStreamer& operator<< (Report::MyStreamer& OS, const PDBAtom & p);

    };


    //___________________________________________________________________PDB

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    PDB(){};
    PDB(size_t n):coords(n,Vector3D(0.0,0.0,0.0)),atoms(n){}
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // New methods of class PDB
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    void clear();
    size_t size() const {return coords.size();}
    void resize(size_t n){coords.resize(n),atoms.resize(n);}
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Vector3DBlock coords;
    std::vector<PDBAtom> atoms;

  };


  //___________________________________________________________________INLINES

}

#endif /* PDB_H */




