/*  -*- c++ -*-  */
#ifndef VECTOR3DBLOCK_H
#define VECTOR3DBLOCK_H

#include <vector>

#include "Vector3D.h"
#include "Proxy.h"


namespace ProtoMol {

  //_________________________________________________________________ Vector3DBlock
  /**
   * Container holding a vector (array) of 3D coordinates/vectors
   */
  class Vector3DBlock: public std::vector<Vector3D>, public Proxy{
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Types & enum's
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
    enum {LIMIT = 30};
  public:
    //typedef std::vector<Vector3D>::size_t size_t;
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    /// Make an empty Vector3DBlock object
    Vector3DBlock(): std::vector<Vector3D>(),Proxy() {}
        
    /// Make a Vector3DBlock with 'size' elements
    Vector3DBlock(size_t size): std::vector<Vector3D>(size),Proxy() {}
    
    /// Make a Vector3DBlock with 'size' elements and initialize them with init.
    Vector3DBlock(size_t size,const Vector3D& init): std::vector<Vector3D>(size,init),Proxy() {}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // New methods of class Vector3DBlock
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    /// Clear (set to zero) each element of the vector.
    void zero(int n=-1) {
      if(n >= 0)
	resize(n);
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i]=Vector3D(0,0,0);
    }

  private:
    /// Catch all clear
    void clear();
  public:
    void cleaR(){std::vector<Vector3D>::clear();}

    //virtual ~Vector3DBlock(){};
    //~Vector3DBlock() { };

    /// copy one Vector3DBlock to another one.
    Vector3DBlock &intoAssign(const Vector3DBlock &x) { 
      //    for(unsigned int i=0;i<size();++i)
      //      (*this)[i] = x[i];
      return *this = x;
    }

    /// Add another Vector3DBlock into this one.
    Vector3DBlock &intoAdd(const Vector3DBlock &x) { 
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i]+=x[i];
      return *this;
    }

    /// Subtract another Vector3DBlock from this one.  
    Vector3DBlock &intoSubtract(const Vector3DBlock &x) {
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i]-=x[i];
      return *this;
    }

    /// Add a scalar multiple of another Vector3DBlock into this one.  
    Vector3DBlock &intoWeightedAdd(Real weight, const Vector3DBlock &x) {
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i] += x[i]*weight;
      return *this;
    }

    /// Add a scalar multiple of another Vector3DBlock into this one.
    Vector3DBlock &intoWeighted(Real weight, const Vector3DBlock &x) {
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i] = x[i]*weight;
      return *this;
    }

    /// Subtract a scalar multiple of another Vector3DBlock from this one.  
    Vector3DBlock &intoWeightedSubtract(Real weight, const Vector3DBlock &x) {
      const unsigned int count = size();
      for(unsigned int i=0;i<count;++i)
	(*this)[i] -= x[i]*weight;
      return *this;
    }

    /// Compute the boinding box over all elements
    void boundingbox(Vector3D& minbb, Vector3D& maxbb) const;

    /// Compute the sum over all elements
    Vector3D sum() const;

    /// Compute regression plane by SVD
    bool fitplane(Vector3D& normal, Real& d, Real& err, int limit = LIMIT) const;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
  };
}

#endif /* VECTOR3DBLOCK_H */
