#include "registerForceExemplarsCutoff.h"
#include "C1SwitchingFunction.h"
#include "C2SwitchingFunction.h"
#include "CellListEnumerator_periodicBoundaries.h"
#include "CellListEnumerator_standard.h"
#include "CoulombForce.h"
#include "CoulombEwaldRealForce.h"
#include "CoulombEwaldRealTableForce.h"
#include "CoulombMultiGridDirectForce.h"
#include "CoulombMultiGridDirectTableForce.h"
#include "CubicCellManager.h"
#include "CutoffSwitchingFunction.h"
#include "ForceFactory.h"
#include "LennardJonesForce.h"
#include "LennardJonesTableForce.h"
#include "MagneticDipoleForce.h"
#include "NonbondedCutoffSystemForce.h"
#include "OneAtomPair.h"
#include "OneAtomPairTwo.h"
#include "PeriodicBoundaryConditions.h"
#include "ShiftSwitchingFunction.h"
#include "Topology.h"
#include "VacuumBoundaryConditions.h"

namespace ProtoMol {
  void registerForceExemplarsCutoff(const PeriodicBoundaryConditions*, const CubicCellManager*){
    // NonbondedCutoffSystemForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,C1SwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,C2SwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,CutoffSwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,ShiftSwitchingFunction,CoulombForce> >());
//     ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
// 				   OneAtomPair<PeriodicBoundaryConditions,UniversalSwitchingFunction,CoulombForce> >());
    
    // NonbondedCutoffSystemForce LennardJonesForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<PeriodicBoundaryConditions,ShiftSwitchingFunction,LennardJonesForce> >());
//     ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
// 				   OneAtomPair<PeriodicBoundaryConditions,UniversalSwitchingFunction,LennardJonesForce> >());

    
    // NonbondedCutoffSystemForce LennardJonesForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,C1SwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,ShiftSwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,ShiftSwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C4> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C4> > >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C4> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<PeriodicBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C4> > >());
                                  
//----------------------------------------------------------------
    // NonbondedCutoffSystemForce LennardJonesForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
                                   OneAtomPairTwo<PeriodicBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
                                   OneAtomPairTwo<PeriodicBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
//----------------------------------------------------------------


    
    // MagneticDipole 
    //    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
    //				   OneAtomPair<PeriodicBoundaryConditions,C1SwitchingFunction,MagneticDipoleForce> >());
  }
  void registerForceExemplarsCutoff(const VacuumBoundaryConditions*, const CubicCellManager*){

    // NonbondedCutoffSystemForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,C1SwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,C2SwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,CutoffSwitchingFunction,CoulombForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,ShiftSwitchingFunction,CoulombForce> >());
//     ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
// 				   OneAtomPair<VacuumBoundaryConditions,UniversalSwitchingFunction,CoulombForce> >());
    
    // NonbondedCutoffSystemForce LennardJonesForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,ShiftSwitchingFunction,LennardJonesForce> >());
//     ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
// 				   OneAtomPair<VacuumBoundaryConditions,UniversalSwitchingFunction,LennardJonesForce> >());
    
    // NonbondedCutoffSystemForce LennardJonesForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,C1SwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,ShiftSwitchingFunction,CoulombForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,ShiftSwitchingFunction,CoulombForce> >());
    
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C4> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectForce<CoulombForce::C4> > >());

    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C1> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C2> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C3> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C1SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C4> > >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPairTwo<VacuumBoundaryConditions,C2SwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombMultiGridDirectTableForce<CoulombForce::C4> > >());

//----------------------------------------------------------------
    // NonbondedCutoffSystemForce LennardJonesForce CoulombForce
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
                                   OneAtomPairTwo<VacuumBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealTableForce> >());
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
                                   OneAtomPairTwo<VacuumBoundaryConditions,CutoffSwitchingFunction,LennardJonesForce,CutoffSwitchingFunction,CoulombEwaldRealForce> >());
//----------------------------------------------------------------

     // MagneticDipole
    ForceFactory::registerExemplar(new NonbondedCutoffSystemForce<CubicCellManager,
				   OneAtomPair<VacuumBoundaryConditions,C1SwitchingFunction,MagneticDipoleForce> >());

  }
}
