/* -*- c++ -*- */
#ifndef COULOMBEWALDREALTABLEFORCE_H
#define COULOMBEWALDREALTABLEFORCE_H

#include "GenericTopology.h"
#include "ScalarStructure.h"
#include "ExclusionTable.h"
#include "Parameter.h"
#include "mathutilities.h"
#include <string>

namespace ProtoMol {

  //_________________________________________________________________ CoulombEwaldRealTableForce
  class CoulombEwaldRealTableForce {
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Replacement potential of the real part for Ewald or PME.
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
    enum {DEFAULT_TABLE_SIZE=20000};
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    CoulombEwaldRealTableForce();
    CoulombEwaldRealTableForce(Real a, Real rc, unsigned int n);
    CoulombEwaldRealTableForce(CoulombEwaldRealTableForce const& other);
    CoulombEwaldRealTableForce& operator=(CoulombEwaldRealTableForce const& other);
    ~CoulombEwaldRealTableForce();
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // New methods of class CoulombEwaldRealTableForce
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    void operator()(Real &energy, 
		    Real &force, 
		    Real distSquared,
		    Real /*rDistSquared*/,
		    const Vector3D &, 
		    const GenericTopology* topo, 
		    int atom1, int atom2,
		    ExclusionClass excl) const {

      Real qq = topo->atoms[atom1].scaledCharge * topo->atoms[atom2].scaledCharge;
      if (topo->coulombScalingFactor != 1.0 && excl == EXCLUSION_MODIFIED) 
	qq *= topo->coulombScalingFactor;
      Real n = sqrt(sqrt(distSquared))*myFac;
      int index = (int)n;
      Real b = n-index;
      //Report::report << index <<","<<sqrt(distSquared);
      index <<=1;
      //Report::report <<","<< index <<","<<myN<<","<<(unsigned long long)myTable<< Report::endr;
      Real a = 1.0-b;
      energy = (myTable[index  ]*a+myTable[index+2]*b)*qq;
      force  = (myTable[index+1]*a+myTable[index+3]*b)*qq;
    }

    static void accumulateEnergy(ScalarStructure* energies, Real energy) {
      (*energies)[ScalarStructure::COULOMB] += energy;
    }

    static Real getEnergy(const ScalarStructure* energies) {return  (*energies)[ScalarStructure::COULOMB];}

    // Parsing
    static std::string getId() {return keyword;}
    static unsigned int getParameterSize() {return 3;}
    void getParameters(std::vector<Parameter>&) const;
    static CoulombEwaldRealTableForce make(std::string&, const std::vector<Value>&);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    static const std::string keyword;      
  private:
    Real* myTable;
    Real myAlpha;
    Real myRc;
    Real myAlphaSquared;
    Real my2AlphaPI;
    Real myFac;
    unsigned int myN;
  };

  //______________________________________________________________________ INLINES
}
#endif /* COULOMBEWALDREALTABLEFORCE_H */
