#include "LennardJonesTableForce.h"
using std::string;
using namespace ProtoMol::Report;
namespace ProtoMol {
  //_________________________________________________________________ LennardJonesTableForce

  const string LennardJonesTableForce::keyword("LennardJonesTable");

  LennardJonesTableForce::LennardJonesTableForce():myTable(NULL),myRc(-1.0),myN(0){}

  LennardJonesTableForce::LennardJonesTableForce(Real rc, unsigned int n):myTable(new Real[4*n]),
											  myRc(rc),
											  myFac(((Real)n-2.0)/rc),
											  myN(n){
    for(unsigned int i=0;i<myN;++i){
      Real r = (i+(i==0?0.01:0.0))/myFac;      

      Real r2   = 1.0/(r*r);
      Real r6   = r2*r2*r2;
      Real r12  = r6*r6;
      //Real r6B  = r6;
      //Real r12A = r12;
      //energy = r12A - r6B;
      //force  = 12.0*r12A*r2 - 6.0*r6B*r2;

      myTable[i*4  ] = r12;
      myTable[i*4+1] = -r6;
      myTable[i*4+2] = 12.0*r12*r2;
      myTable[i*4+3] = -6.0*r6*r2;
    }
    report <<hint<<"Using LennardJones approximation with "<<n<<" points, range ["<<(1.0/myFac)<<","<<((Real)n/myFac)<<"[, table[0,1,"<<myN-1<<"] = ("<<myTable[0]<<","<<myTable[1]<<"),("<<myTable[2]<<","<<myTable[3]<<"),("<<myTable[(int)myN*4-2]<<","<<myTable[(int)myN*4-1]<<")."<<endr;
  }

  LennardJonesTableForce::~LennardJonesTableForce(){
    if(myTable != NULL)
      delete [] myTable;
    myTable = NULL;
  }

  LennardJonesTableForce::LennardJonesTableForce(LennardJonesTableForce const& other):myTable(new Real[4*other.myN]),
												  myRc(other.myRc),
												  myFac(((Real)other.myN-2.0)/other.myRc),
												  myN(other.myN){
    for(unsigned int i=0;i<myN*4;++i)
      myTable[i] = other.myTable[i];    
  }

  LennardJonesTableForce& LennardJonesTableForce::operator=(LennardJonesTableForce const& other){
    if(&other != this){
      if(myTable != NULL)
	delete [] myTable;
      myRc           = other.myRc;
      myFac          = other.myFac;
      myN            = other.myN;
      myTable        = new Real[4*myN];
      for(unsigned int i=0;i<myN*4;++i)
	myTable[i] = other.myTable[i];    
    }
    return *this;
  }

  void LennardJonesTableForce::getParameters(std::vector<Parameter>& parameters) const{
    parameters.push_back(Parameter("-tableCutoff",Value(myRc,ConstraintValueType::Positive()),Text("cutoff for table look up")));
    parameters.push_back(Parameter("-tableSize",Value(myN,ConstraintValueType::Positive()),static_cast<unsigned int>(DEFAULT_TABLE_SIZE),Text("size of table look up")));
  }

  LennardJonesTableForce LennardJonesTableForce::make(std::string& , const std::vector<Value>& values) {
    return LennardJonesTableForce(values[0],values[1]);
  }

}
