/* -*- c++ -*- */
#ifndef LENNARDJONESTABLEFORCE_H
#define LENNARDJONESTABLEFORCE_H

#include "GenericTopology.h"
#include "ScalarStructure.h"
#include "ExclusionTable.h"
#include "Parameter.h"
#include "mathutilities.h"
#include <string>

namespace ProtoMol {

  //_________________________________________________________________ LennardJonesTableForce
  class LennardJonesTableForce {
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Replacement potential of the real part for  or PME.
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
    enum {DEFAULT_TABLE_SIZE=20000};
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    LennardJonesTableForce();
    LennardJonesTableForce(Real rc, unsigned int n);
    LennardJonesTableForce(LennardJonesTableForce const& other);
    LennardJonesTableForce& operator=(LennardJonesTableForce const& other);
    ~LennardJonesTableForce();
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // New methods of class LennardJonesTableForce
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    void operator()(Real &energy, 
		    Real &force, 
		    Real distSquared,
		    Real /*rDistSquared*/,
		    const Vector3D &, 
		    const GenericTopology* topo, 
		    int atom1, int atom2,
		    ExclusionClass excl) const {

      const LennardJonesParameters &
	params=topo->lennardJonesParameters(topo->atoms[atom1].type,topo->atoms[atom2].type);

      Real A = (excl != EXCLUSION_MODIFIED ? params.A : params.A14);
      Real B = (excl != EXCLUSION_MODIFIED ? params.B : params.B14); 

      Real n = sqrt(distSquared)*myFac;
      int index = (int)n;
      Real b = n-index;
      //Report::report << index <<","<<sqrt(distSquared);
      index <<=2;
      //Report::report <<","<< index <<","<<myN<<","<<(unsigned long long)myTable<< Report::endr;
      Real a = 1.0-b;
      energy = (myTable[index  ]*a+myTable[index+4]*b)*A + (myTable[index+1]*a+myTable[index+5]*b)*B;
      force  = (myTable[index+2]*a+myTable[index+6]*b)*A + (myTable[index+3]*a+myTable[index+7]*b)*B;
    }

    static void accumulateEnergy(ScalarStructure* energies, Real energy) {
      (*energies)[ScalarStructure::LENNARDJONES] += energy;
    }

    static Real getEnergy(const ScalarStructure* energies) {return  (*energies)[ScalarStructure::LENNARDJONES];}

    // Parsing
    static std::string getId() {return keyword;}
    static unsigned int getParameterSize() {return 2;}
    void getParameters(std::vector<Parameter>&) const;
    static LennardJonesTableForce make(std::string&, const std::vector<Value>&);
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    static const std::string keyword;      
  private:
    Real* myTable;
    Real myRc;
    Real myFac;
    unsigned int myN;
  };

  //______________________________________________________________________ INLINES
}
#endif /* LENNARDJONESTABLEFORCE_H */
