#include "OutputDiffusion.h"
#include "GenericTopology.h"
#include "topologyutilities.h"
#include "OutputCache.h"
#include "Integrator.h"
#include "inputValueDefinitions.h"

#include <iomanip>

using namespace ProtoMol::Report;

using std::string;
using std::vector;
using std::setw;
using std::endl;
using std::flush;
using std::stringstream;
using std::setprecision;
using std::setiosflags;
using std::resetiosflags;
using std::ofstream;

namespace ProtoMol {
  //________________________________________________________ Output
  const string  OutputDiffusion::keyword("diffusionFile");

  OutputDiffusion::OutputDiffusion(): OutputFile(){}
  
  OutputDiffusion::OutputDiffusion(const string& filename, int freq): OutputFile(filename,freq){}

  void OutputDiffusion::doInitialize(){
    ofstream diffusionHeaderFile(string(myFilename + ".header").c_str(), std::ios::out | std::ios::trunc);
    if(!diffusionHeaderFile)
      report << error <<" Can not open \'"<<myFilename<<".header\' for "<<getId()<<"."<<endr;

    diffusionHeaderFile << setw(18)
			<< "Time(fs)" << " "
			<< setw(24)
			<< "Diffusion(10^-5cm^2/s)" << " "
			<< setw(14)
			<< "Volume(AA^3)" << " "
			<< setw(14)
			<< "Temperature(K)" << " "
			<< setw(14)
			<< "Density(kgm^-3*1e3)";
    diffusionHeaderFile << endl;
    diffusionHeaderFile.close();
    open();
    close();

  }
  
  void OutputDiffusion::doRun(int step){
    if(!reopen())
      report << error <<" Can not open \'"<<myFilename<<"\' for "<<getId()<<"."<<endr;
    Real diffusion = 10000.0*myCache->diffusion()*(first()?0.0:1.0/((step-getFirstStep())*myIntegrator->getTimestep()));

    myFile << resetiosflags(std::ios::showpoint |  std::ios::fixed | std::ios::floatfield)
	   << setw(18)
	   << setprecision(2)
	   << setiosflags(std::ios::showpoint |
			  std::ios::fixed)
	   << myCache->time() << " "
	   << resetiosflags(std::ios::showpoint |
			    std::ios::fixed)
	   << setiosflags(std::ios::floatfield)
	   << setprecision(8)
	   << setw(24)
	   << diffusion << " "
	   << setw(14)
	   << myCache->volume()<< " "
	   << setw(14)
	   << myCache->temperature() << " "
	   << setw(14)
	   << myCache->density();
    myFile << endl;
    reclose();
  }

  void OutputDiffusion::doFinalize(int){
    close();
  }

  Output* OutputDiffusion::doMake(string&, const vector<Value>& values) const{
    return (new OutputDiffusion(values[0],values[1]));
  }

  void OutputDiffusion::getParameters(vector<Parameter> &parameter) const{
    parameter.push_back(Parameter(getId(),Value(myFilename,ConstraintValueType::NotEmpty())));
    parameter.push_back(Parameter("diffusionOutputFreq",Value(myOutputFreq,ConstraintValueType::Positive())));
  }

  bool OutputDiffusion::adjustWithDefaultParameters(vector<Value>& values, const Configuration* config) const{
    if(!checkParameterTypes(values))
      return false;
    if(config->valid(InputOutputfreq::keyword) && !values[1].valid())
      values[1] = (*config)[InputOutputfreq::keyword];
    return checkParameters(values);
  }

}
