#include "OutputMomentum.h"
#include "GenericTopology.h"
#include "topologyutilities.h"
#include "OutputCache.h"
#include "inputValueDefinitions.h"

#include <iomanip>

using namespace ProtoMol::Report;

using std::string;
using std::vector;
using std::setw;
using std::endl;
using std::flush;
using std::stringstream;
using std::setprecision;
using std::setiosflags;
using std::resetiosflags;
using std::ofstream;

namespace ProtoMol {
  //________________________________________________________ Output
  const string  OutputMomentum::keyword("momentumFile");

  OutputMomentum::OutputMomentum(): OutputFile(){}
  
  OutputMomentum::OutputMomentum(const string& filename, int freq): OutputFile(filename,freq){}

  void OutputMomentum::doInitialize(){
    ofstream momentumHeaderFile(string(myFilename + ".header").c_str(), std::ios::out | std::ios::trunc);
    if(!momentumHeaderFile)
      report << error <<" Can not open \'"<<myFilename<<".header\' for "<<getId()<<"."<<endr;

    momentumHeaderFile << setw(14)
			  << "Time(fs)" << " "
			  << setw(25)
			  << "10^6*LinMomentum_X" << " "
			  << setw(25)
			  << "10^6*LinMomentum_Y" << " "
			  << setw(25)
			  << "10^6*LinMomentum_Z" << " "
 			  << setw(25)
			  << "10^6*AngMomentum_X" << " "
			  << setw(25)
			  << "10^6*AngMomentum_Y" << " "
			  << setw(25)
			  << "10^6*AngMomentum_Z";
   momentumHeaderFile << endl;
    momentumHeaderFile.close();
    open();
    close();

  }
  
  void OutputMomentum::doRun(int){
    if(!reopen())
      report << error <<" Can not open \'"<<myFilename<<"\' for "<<getId()<<"."<<endr;
    
    Real f = 1e6*Constant::INV_TIMEFACTOR;
    
    myFile << resetiosflags(std::ios::showpoint |  std::ios::fixed | std::ios::floatfield)
	   << setw(14)<< setprecision(2)
	   << setiosflags(std::ios::showpoint | std::ios::fixed)
	   << myCache->time() << " "
	   << setw(25) << setprecision(5) << myCache->linearMomentum().x*f<< " "
	   << setw(25) << setprecision(5) << myCache->linearMomentum().y*f<< " "
	   << setw(25) << setprecision(5) << myCache->linearMomentum().z*f<< " "
	   << setw(25) << setprecision(5) << myCache->angularMomentum().x*f<< " "
	   << setw(25) << setprecision(5) << myCache->angularMomentum().y*f<< " "
	   << setw(25) << setprecision(5) << myCache->angularMomentum().z*f;
    myFile << endl;
    reclose();
  }

  void OutputMomentum::doFinalize(int){
    close();
  }

  Output* OutputMomentum::doMake(string&, const vector<Value>& values) const{
    return (new OutputMomentum(values[0],values[1]));
  }

  void OutputMomentum::getParameters(vector<Parameter> &parameter) const{
    parameter.push_back(Parameter(getId(),Value(myFilename,ConstraintValueType::NotEmpty())));
    parameter.push_back(Parameter("momentumOutputFreq",Value(myOutputFreq,ConstraintValueType::Positive())));
  }

  bool OutputMomentum::adjustWithDefaultParameters(vector<Value>& values, const Configuration* config) const{
    if(!checkParameterTypes(values))
      return false;
    if(config->valid(InputOutputfreq::keyword) && !values[1].valid())
      values[1] = (*config)[InputOutputfreq::keyword];
    return checkParameters(values);
  }

}
