#include "HMCIntegrator.h"
#include "Report.h"
#include "mathutilities.h"
#include "pmconstants.h"
#include "Vector3DBlock.h"
#include "ScalarStructure.h"
#include "topologyutilities.h"

using namespace ProtoMol::Report;
using std::vector;
using std::string;

namespace ProtoMol {
  //____________________________________________________________ HMCIntegrator

  const string HMCIntegrator::keyword("HybridMC");

  const int HMCIntegrator::myNumParameters(3);

  //  ---------------------------------------------------------------------  //

  HMCIntegrator::HMCIntegrator()    
    : MCIntegrator() {}

  //  ---------------------------------------------------------------------  //

  HMCIntegrator::HMCIntegrator(int cycles,
			       bool randomCycLen,
			       Real initialTemperature,
			       ForceGroup *overloadedForces,
			       StandardIntegrator *nextIntegrator)
    
    : MCIntegrator(cycles, randomCycLen, initialTemperature, overloadedForces,
            nextIntegrator) {
  }
  
  //  ---------------------------------------------------------------------  //

  void HMCIntegrator::initialize(GenericTopology *topo,
				 Vector3DBlock *positions,
				 Vector3DBlock *velocities,
				 ScalarStructure *energies){
    MCIntegrator::initialize(topo,positions,velocities,energies);
  }

  //  ---------------------------------------------------------------------  //

  MTSIntegrator*  HMCIntegrator::doMake(string& , const vector<Value>& values, 
					ForceGroup* fg, StandardIntegrator *nextIntegrator)const{

      return new HMCIntegrator(values[0],values[1],values[2],fg,nextIntegrator);

  }

  //  ---------------------------------------------------------------------  //
  
  void HMCIntegrator::perturbSystem() {
    randomVelocity(getInitialTemperature(), myTopo,myVelocities);
    buildMolecularMomentum(myVelocities,myTopo);
  }

  //  ---------------------------------------------------------------------  //
  
  void HMCIntegrator::saveValues(){
    MCIntegrator::saveValues();
  }

  //  ---------------------------------------------------------------------  //
  
  void HMCIntegrator::restoreValues(){
    MCIntegrator::restoreValues();   
  }

}
