/*  -*- c++ -*-  */
#ifndef SHADOWHMCINTEGRATOR_H
#define SHADOWHMCINTEGRATOR_H

#include "MCIntegrator.h"
#include <deque>

namespace ProtoMol {
  class GenericTopology;
  class ScalarStructure;
  class Vector3DBlock;
  class ForceGroup;
  class ModifierUpdateBeta;
  class ModifierUpdateBetaAndPush;
  //____________________________________________________________ ShadowHMCIntegrator

  class ShadowHMCIntegrator: public MCIntegrator {

    friend class ModifierUpdateBeta;
    friend class ModifierUpdateBetaAndPush;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    ShadowHMCIntegrator();  
    ShadowHMCIntegrator(int cycles,
			bool randomCycLen,
			Real initialTemperature,
			unsigned int shadowOrder,
			Real c,
			ForceGroup *overloadedForces,
			StandardIntegrator *nextIntegrator);
    virtual ~ShadowHMCIntegrator();  



    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // New methods of class ShadowHMCIntegrator
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
    void updateBeta();
    void pushShadowHistory();
    void calculateShadow();
    Real calcShadow4();
    Real calcShadow8();
    void runPreSteps(int numTimesteps);  
    void runPostSteps(int numTimesteps);  
    void resetHistory();   
    void disableBetaUpdate();
    void enableBetaUpdate();


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From class Makeable
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    virtual std::string getIdNoAlias() const{return keyword;}
    virtual unsigned int getParameterSize() const{return 5;}
    virtual void getParameters(std::vector<Parameter>& parameters) const;


    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From class Integrator
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    virtual void initialize(GenericTopology *topo,
			    Vector3DBlock *positions,
			    Vector3DBlock *velocities,
			    ScalarStructure *energies);

    virtual void run(int numTimesteps);
  
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From class MTSIntegrator
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
    virtual MTSIntegrator* doMake(std::string& errMsg,
                const std::vector<Value>& values, ForceGroup* fg,
                StandardIntegrator *nextIntegrator) const;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From classs MCIntegrator
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  protected:
    virtual void perturbSystem();
    virtual void saveValues();
    virtual void restoreValues();

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    static const std::string keyword;

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:
    unsigned int myOrder,
                 myShadowK,
                 myShadowKover2;

    Real myC;  
    Real myBeta;

    Real sumTotalEnergy;

    std::deque< Vector3DBlock > *myPrevVelocities; 
    std::deque< Vector3DBlock > *myPrevPositions;
    std::deque< Real >          *myPrevBeta;  

    ModifierUpdateBeta* myModifier1;
    ModifierUpdateBetaAndPush* myModifier2;

    Vector3DBlock*   myOldForces;
    
  };
}
#endif
