#include "UmbrellaIntegrator.h"
#include "Modifier.h"
#include "Vector3DBlock.h"
#include "Report.h"
#include "ForceGroup.h"
#include "Force.h"

#include <algorithm>

using namespace ProtoMol::Report;
using std::vector;
using std::string;

namespace ProtoMol {



  class ModifierUmbrella : public Modifier {
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // Constructors, destructors, assignment
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    ModifierUmbrella():Modifier(Constant::MAX_INT-200){}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // From class Modifier
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  public:
    virtual bool isInternal() const {return true;}
  private:
    virtual void doExecute();

    virtual std::string doPrint()const{return std::string("Umbrella");}

    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    // My data members
    //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  private:

  };

  void ModifierUmbrella::doExecute(){
    report << myTopology->time << endr;
  }


  //_________________________________________________________ UmbrellaIntegrator

  const string UmbrellaIntegrator::keyword("Umbrella");

  const int UmbrellaIntegrator::myNumParameters(1);

  //  ---------------------------------------------------------------------  //  

  UmbrellaIntegrator::UmbrellaIntegrator():MTSIntegrator(){}

  UmbrellaIntegrator::UmbrellaIntegrator (int cycles,
					  ForceGroup *overloadedForces,
					  StandardIntegrator *nextIntegrator)
    : MTSIntegrator(cycles,overloadedForces,nextIntegrator)
  {
    hdf = NULL;
    curAngle = 0;
  }
  
  //  ---------------------------------------------------------------------  //
  
  UmbrellaIntegrator::~UmbrellaIntegrator(){}

  //  ---------------------------------------------------------------------  //  

  MTSIntegrator*  UmbrellaIntegrator::doMake(string& ,
					     const vector<Value>& values,
					     ForceGroup* fg,
					     StandardIntegrator *nextIntegrator)const{
    return new UmbrellaIntegrator(values[0],fg,nextIntegrator);
  }
  //  ---------------------------------------------------------------------  //
      
  void UmbrellaIntegrator::getParameters(vector< Parameter> &parameters) const {
    MTSIntegrator::getParameters(parameters);
    //parameters.push_back(Parameter( "randomCycLen", Value( myRandomCycLen ), false ) );
  }     
  
  //  ---------------------------------------------------------------------  //
  
  void UmbrellaIntegrator::initialize(GenericTopology *topo,
				      Vector3DBlock *positions,
				      Vector3DBlock *velocities,
				      ScalarStructure *energies){
    MTSIntegrator::initialize(topo,positions,velocities,energies);

    std::vector<Force*> myForcePtrs = next()->getForceGroup()->getForces();
    //std::cout << "myForce Ptrs size: " << myForcePtrs.size();
    std::vector<Force*>::iterator forceItr = myForcePtrs.begin();
    while( (forceItr != myForcePtrs.end()) && (hdf == 0))
      {
	std::cout << (*forceItr)->getId() << std::endl;
	if(equalNocase((*forceItr)->getId(),"HarmDihedral"))
	  hdf = (*forceItr);
	forceItr++;
      }
    if (hdf == 0)
      {
	std::cout << "A HarmDihedralSystemForce pointer was not found" << std::endl;      
	report << debug(1) << "A HarmDihedralSystemForce pointer was not found" << endr;

      }
    else
      {
	std::cout << "hdf points to a force of type: ";
	std::cout << hdf->getId() << std::endl;
	std::cout.flush();
	report << debug(1) << hdf->getId() << ":" << hdf->getParameters()[1].value << endr;
      }  

    curAngle = hdf->getParameters()[1].value;

    this->bottom()->adoptPostForceModifier(new ModifierUmbrella());
  }

  //  ---------------------------------------------------------------------  //
  
  void UmbrellaIntegrator::run(int numTimesteps) {

    for(int i = 0; i < numTimesteps; i++) {

      incrCoord();

      myNextIntegrator->run(myCycleLength);
      
    }  

  }

  //  ---------------------------------------------------------------------  //  
  
  void UmbrellaIntegrator::incrCoord(){

    curAngle = curAngle + 10;
    if (curAngle > 180)
      {
	curAngle = curAngle - 360;
      }
    
    std::string err;
    std::cout << "Trial angle for lift: " << curAngle << std::endl;
    //hdf->setParameter(err,0,0);
    hdf->setParameter(err,1,curAngle);
    report << debug(1) << hdf->getId() << ":" << hdf->getParameters()[0].value
           << ":" << hdf->getParameters()[1].value << endr;
  }



  
}
