#include "ExclusionTable.h"

namespace ProtoMol {
  //_________________________________________________________________ ExclusionTable
  
  void ExclusionTable::resize(int count) {
    lowDeltas.resize(count*Constant::FASTDELTAMAX);
    anyExclsForThisDelta.resize(count);
  }
  
  void ExclusionTable::cleanTemporaries(){
    mySet.clear();
  }

  void ExclusionTable::clear(void) {
    for(unsigned int i=0;i<lowDeltas.size();i++)
      lowDeltas[i]=EXCLUSION_NONE;
    highDeltas.clear();
    for(unsigned int i=0;i<anyExclsForThisDelta.size();i++)
      anyExclsForThisDelta[i]=false;
    myMaxDelta=0;
    myTable.resize(0);
    mySet.clear();
  }
  
  void ExclusionTable::add(int atom1, int atom2, ExclusionClass type) {
    if(atom1>atom2) 
      std::swap(atom1,atom2);
    int delta=atom2-atom1;
    if(type != EXCLUSION_NONE && atom1 != atom2 && mySet.find(ExclusionPair(atom1,atom2,type)) == mySet.end()){
      myTable.push_back(ExclusionPair(atom1,atom2,type));
      mySet.insert(ExclusionPair(atom1,atom2,type));
    }
    if(delta>myMaxDelta) 
      myMaxDelta=delta;
    if(delta<Constant::FASTDELTAMAX) {
      lowDeltas[atom1*Constant::FASTDELTAMAX+delta]=type;
    } else {
      highDeltas[PairInt(atom1,delta)]=type;
      anyExclsForThisDelta[delta]=true;
    }
  }
}
